#!/usr/bin/perl
# $Id: createGraph.pl,v 1.10 2010/01/08 07:42:20 zahorjan Exp $

use strict;
use Getopt::Long;

my $useBestFit;
GetOptions('wbf'=>\$useBestFit);

die "Usage: createGraph.pl dataFilename [pdf | x]" unless @ARGV > 0;

my $filename = shift;
$filename = "driverResultsFiles/$filename" unless -r $filename;
$filename = `ls $filename*` unless -r $filename;
open(INFILE, "<$filename") || die "Can't read $filename";

my $plotType = 'x11';
$plotType = 'postscript color' if $ARGV[0] eq 'pdf';
die "Unrecognized output arg $ARGV[0]" if $ARGV[0] && $ARGV[0] ne 'pdf';


my $dataHost = `basename $filename`;
chomp $dataHost;
$dataHost = $1 if $dataHost =~ m/^([^\.]+)/;

# Extract data from the driver data file

my @data;
my $dataSet;
while(<INFILE>) {
  push(@data, $dataSet = { title => $1, data => []}) if m/^--- (.*)/;
  $dataSet->{data}->[@{$dataSet->{data}}][0] = sprintf("%f", $1) if m/^Processed (\d+\.\d*(E\d+)?)/;
  $dataSet->{data}->[$#{$dataSet->{data}}][1] = sprintf("%f", $1) if m/^Goodput: (\d+\.\d*)/;
}
close(INFILE);

# Create gnuplot input file

my $gpDataFile = `mktemp gpdata.XXXXXXXX`;
chomp $gpDataFile;

open (OUTFILE, ">$gpDataFile") || die "Couldn't create temp file $gpDataFile";

my $waitForUserInput = '-';  # used in gnuplot invocation (down below) to allow
                             # user to decide when to end the gnuplot session
my $setOutputLine = '';
my $gpPSFile;
if ( $plotType =~ m/^postscript/ ) {
    $gpPSFile = "$gpDataFile.ps";
    #print "Writing temporary postscript file '$gpPSFile'\n";
    $setOutputLine = "set output '$gpPSFile'";  # cause gnuplot to write a file
    $waitForUserInput = '';
}

my @fitList;
foreach my $f (@data) {
  $f->{data} = [grep{ $_->[0] > 0 } @{$f->{data}}];
  next unless $useBestFit && $f->{title} =~ /\(([^:]+):(\d+)\)/;

  $f->{fitTitle} = "Best Fit $1:$2";
  my $i = 1 + $#fitList;
  printf OUTFILE "f$i(x) = x / (a$i + b$i * x)
                 fit f$i(x) '-' via a$i, b$i\n";
  printf OUTFILE join("\n", map{ "$_->[0] $_->[1]"} @{$f->{data}} );
  printf OUTFILE "\ne\n";
  $f->{fitFunc} = "f$i(x)";
  push(@fitList, $f);
}

my $plotCmd = 'plot ' .
              join(', ', map{ "'-' title '$_->{title}' with linespoints"; } @data );
if (@fitList) {
  $plotCmd .= ', ' .join(', ', map { " $_->{fitFunc} title '$_->{fitTitle}'"; } @fitList);
}
printf OUTFILE "\n";


printf OUTFILE <<HEADER_END;
set terminal $plotType
$setOutputLine
set title 'Goodput Results $dataHost'
set xlabel 'Xfer size'
set ylabel 'Goodput (Mbps)'
set key box top left
set multiplot
set logscale x 10
$plotCmd
HEADER_END

foreach $dataSet ( @data ) {
  printf OUTFILE join("\n", map{"$_->[0] $_->[1]"} grep { $_->[0] > 0 } @{$dataSet->{data}});
  printf OUTFILE "\ne\n";  # 'e' ends the list of coordinates
}

printf OUTFILE "unset multiplot\n";

close(OUTFILE);

#---------------------------------------------------------------
# Try to run gnuplot
#---------------------------------------------------------------
my $gpResult = system("gnuplot $gpDataFile $waitForUserInput");
# if the result isn't zero, something went wrong
die "gnuplot invocation failed: rcode = $gpResult" if $gpResult;
system("rm -f $gpDataFile");

#---------------------------------------------------------------
# Convert to pdf, if requested
#---------------------------------------------------------------
if ( $plotType =~ m/^postscript/ ) {
    my $pdfFile = $dataHost . '.pdf';
    my $psResult = system("ps2pdf $gpPSFile $pdfFile");  # invoke the ps2pdf program
    die "ps2pdf invocation failed: rcode = $gpResult" if $gpResult;
    system("rm $gpPSFile");
    print "Output file is $pdfFile\n";
}

